%{
    #include "verilog_parser.tab.h" 
	#include "object.h"
	#include "preprocess.h"   
    #include "verilog_preprocess.h"

    static IPreprocess ** preprocess = NULL;
	static char * __macroname = NULL;

	int SetPreProcess(HOBJECT object)
	{
		return objectQueryInterface(object, IID_PREPROCESS, (const void **)&preprocess);
	}

	extern int verilog_find_reserved_word(const char * ident);
	#define YY_NO_UNISTD_H

	#define YY_INPUT(buf, result, max_size) \
	{ \
		 result = objectCall2(preprocess, GetText, buf, max_size); \
	}

    #define EMIT_TOKEN(x) if (objectCall0(preprocess, SymbolEmitEnabled)) { return x; }
%}

%option outfile="verilog_scanner.c"
%option yylineno
%option nodefault 
%option noyywrap 

/* Pre-processor definitions */
CD_DEFAULT_NETTYPE     "`default_nettype"
%x in_default_nettype

CD_LINE                "`line"
%x in_line_1
%x in_line_2
%x in_line_3
%x in_line_4

CD_CELLDEFINE          "`celldefine"
CD_DEFINE              "`define"
CD_RESETALL            "`resetall"
CD_ENDCELLDEFINE       "`endcelldefine"

CD_ELSE                "`else"
CD_ELSIF               "`elsif"
CD_ENDIF               "`endif"
CD_IFDEF               "`ifdef"
CD_IFNDEF              "`ifndef"

%x in_ifdef
%x in_ifndef
%x in_elseif

CD_UNDEF               "`undef"

%x in_undef

CD_NOUNCONNECTED_DRIVE "`nounconnected_drive"
CD_UNCONNECTED_DRIVE   "`unconnected_drive"

%x in_unconnected_drive

/* Include Directives */

CD_INCLUDE             "`include"

%x in_include

/* Times and compiler directives */

CD_TIMESCALE           "`timescale"
TIME_UNITS             "(s|ms|us|ns|ps|fs)"
%x in_ts_1
%x in_ts_2
%x in_ts_3


/* Single character tokens */

NEWLINE             "\n"|"\r\n"
SPACE               " "
TAB                 "\t"

AT                  "@"
COMMA               ","
HASH                "#"
DOT                 "."
EQ                  "="
COLON               ":"
IDX_PRT_SEL         "+:"|"-:"
SEMICOLON           ";"
OPEN_BRACKET        "\("
CLOSE_BRACKET       "\)"
OPEN_SQ_BRACKET     "\["
CLOSE_SQ_BRACKET    "\]"
OPEN_SQ_BRACE       "{"
CLOSE_SQ_BRACE      "}"


/* Tokens related to numbers */

EXP                 "e"|"E"
UNDERSCORE          "_"
SIGN                {PLUS}|{MINUS}
X                   "x"|"X"
Z                   "z"|"Z"|"?"

DIGIT_DECIMAL       [0-9]
DIGIT_DECMIAL_NZ    [1-9]
DIGIT_BINARY        [0-1]|{X}|{Z}
DIGIT_OCTAL         [0-7]|{X}|{Z}
DIGIT_HEX           [0-9a-fA-F]|{X}|{Z}
DIGIT_DEC			[0-9]|{X}|{Z}

BASE_DECIMAL        '[sS]?[dD]
BASE_BINARY         '[sS]?[bB]
BASE_OCTAL          '[sS]?[oO]
BASE_HEX            '[sS]?[hH]

NUM_REAL_EXP        {NUM_UNSIGNED}({DOT}{NUM_UNSIGNED})?{EXP}({SIGN})?{NUM_UNSIGNED}

BIN_VALUE           {DIGIT_BINARY}({UNDERSCORE}|{DIGIT_BINARY})*
OCT_VALUE           {DIGIT_OCTAL}({UNDERSCORE}|{DIGIT_OCTAL})*
HEX_VALUE           {DIGIT_HEX}({UNDERSCORE}|{DIGIT_HEX})*
DEC_VALUE           {DIGIT_DEC}({UNDERSCORE}|{DIGIT_DEC})*

%x in_dec_val
%x in_hex_val
%x in_oct_val
%x in_bin_val
%s in_number

NUM_REAL            {NUM_UNSIGNED}{DOT}{NUM_UNSIGNED}|{NUM_REAL_EXP}
NUM_UNSIGNED        {DIGIT_DECIMAL}({UNDERSCORE}|{DIGIT_DECIMAL})*

/* Identifiers */

SYSTEM_ID           \$[a-zA-Z0-9_\$]+
SIMPLE_ID           [a-zA-Z_][a-zA-Z0-9_$]*
ESCAPED_ID          \\{SIMPLE_ID}
MACRO_IDENTIFIER    `{SIMPLE_ID}

MACRO_TEXT          .*\n

%x in_define
%x in_define_t

/* Attributes */

ATTRIBUTE_START     \(\*
ATTRIBUTE_END       \*\)

/* Comments */
COMMENT_LINE        "//".*\n

COMMENT_BEGIN       "/*"
COMMENT_END         "*/"

%x in_comment

/* Strings */

STRING              \".*\"

/* Operators */

STAR                "\*"
PLUS                "+"
MINUS               "-"
ASL                 "<<<"
ASR                 ">>>"
LSL                 "<<"
LSR                 ">>"
DIV                 "/"
POW                 "**"
MOD                 "%"
GTE                 ">="
LTE                 "<="
GT                  ">"
LT                  "<"
L_NEG               "!"
L_AND               "&&"
L_OR                "||"
C_EQ                "==="
L_EQ                "=="
C_NEQ               "!=="
L_NEQ               "!="
B_NEG               "~"
B_AND               "&"
B_OR                "|"
B_XOR               "^"
B_EQU               "^~"|"~^"
B_NAND              "~&"
B_NOR               "~|"
TERNARY             "?"

%%
{ATTRIBUTE_START}      {EMIT_TOKEN(ATTRIBUTE_START);}
{ATTRIBUTE_END}        {EMIT_TOKEN(ATTRIBUTE_END);}

{COMMENT_LINE}         {/*EMIT_TOKEN(COMMENT_LINE); IGNORE */}
{COMMENT_BEGIN}        {BEGIN(in_comment);                    ;}

<in_comment>.|\n       {/* IGNORE                            */}
<in_comment>{COMMENT_END} {BEGIN(INITIAL);                     }

{CD_CELLDEFINE}          {/*verilog_preproc_enter_cell_define();*/}
{CD_ENDCELLDEFINE}       {/*verilog_preproc_exit_cell_define();*/}

{CD_DEFAULT_NETTYPE}     {BEGIN(in_default_nettype);}
<in_default_nettype>{SIMPLE_ID}  {
	objectCall3(preprocess, PreAction, PA_NETTYPE, yytext, "");
    BEGIN(INITIAL); 
    }

{CD_TIMESCALE}           {
    BEGIN(in_ts_1);
}
<in_ts_1>{NUM_UNSIGNED}      {
    objectCall2(preprocess, SetParam, "timescale.scale", yytext);
}
<in_ts_1>{SIMPLE_ID}         {
    BEGIN(in_ts_2);
}
<in_ts_2>{DIV}               {
    BEGIN(in_ts_3);
}
<in_ts_3>{NUM_UNSIGNED}      {
	objectCall2(preprocess, SetParam, "timescale.precision", yytext);
}
<in_ts_3>{SIMPLE_ID}         {
    BEGIN(INITIAL);
}

{CD_RESETALL}            {
    /*verilog_preprocessor_resetall();*/
}

{CD_IFDEF}               {
    BEGIN(in_ifdef);
}
<in_ifdef>{SIMPLE_ID}    {
	objectCall3(preprocess, PreAction, PA_IFDEF, yytext, "");
    BEGIN(INITIAL);
}

{CD_IFNDEF}              {
    BEGIN(in_ifndef);
}
<in_ifndef>{SIMPLE_ID}   {
	objectCall3(preprocess, PreAction, PA_IFNDEF, yytext, "");
    BEGIN(INITIAL);
}

{CD_ELSIF}               {
    BEGIN(in_elseif);
}
<in_elseif>{SIMPLE_ID}   {
	objectCall3(preprocess, PreAction, PA_ELSEIFDEF, yytext, "");
    BEGIN(INITIAL);
}

{CD_ELSE}                {
	objectCall3(preprocess, PreAction, PA_ELSE, "", "");
}

{CD_ENDIF}               {
	objectCall3(preprocess, PreAction, PA_ENDIF, "", "");
}

{CD_INCLUDE}             {
    BEGIN(in_include);
}
<in_include>{STRING}     {
	if (0 == objectCall3(preprocess, PreAction, PA_INCLUDE, yytext, "")) {
		yypush_buffer_state(yy_create_buffer(stdin, YY_BUF_SIZE));
	}
    BEGIN(INITIAL);
}

{CD_LINE}                 {BEGIN(in_line_1);}
<in_line_1>{NUM_UNSIGNED} {BEGIN(in_line_2);}
<in_line_2>{STRING}       {BEGIN(in_line_3);}
<in_line_3>{NUM_UNSIGNED} {BEGIN(INITIAL);}

{CD_NOUNCONNECTED_DRIVE} {
	objectCall2(preprocess, SetParam, "nounconnected_drive", "NONE");
}
{CD_UNCONNECTED_DRIVE}   {
    BEGIN(in_unconnected_drive);
}
<in_unconnected_drive>{SIMPLE_ID} {
	objectCall2(preprocess, SetParam, "nounconnected_drive", yytext);
    BEGIN(INITIAL);
}

{CD_DEFINE}              {
    BEGIN(in_define);
}

<in_define>{SIMPLE_ID}   {
    __macroname = strdup(yytext);
    BEGIN(in_define_t);
}

<in_define_t>{MACRO_TEXT} {
    if(yyleng == 1)
    {
        // Macro has no value, and is just a newline character.
		objectCall3(preprocess, PreAction, PA_DEFINE, __macroname, "");
    }
    else
    {
		objectCall3(preprocess, PreAction, PA_DEFINE, __macroname, yytext+1);
    }
	free(__macroname);
    BEGIN(INITIAL);
}

{CD_UNDEF}             {
    BEGIN(in_undef);
}

<in_undef>{SIMPLE_ID}  {
	objectCall3(preprocess, PreAction, PA_UNDEF, yytext, "");
    BEGIN(INITIAL);
}

{MACRO_IDENTIFIER}     {
	if (0 == objectCall3(preprocess, PreAction, PA_MACRO, (yytext)+1, "")) {
		yypush_buffer_state(yy_create_buffer(stdin, YY_BUF_SIZE));
	}
}

{AT}                   {EMIT_TOKEN(AT);}
{COMMA}                {EMIT_TOKEN(COMMA);}
{HASH}                 {EMIT_TOKEN(HASH);}
{DOT}                  {EMIT_TOKEN(DOT);}
{EQ}                   {yylval.operator = OPERATOR_L_EQ; EMIT_TOKEN(EQ);}
{COLON}                {EMIT_TOKEN(COLON);}
{IDX_PRT_SEL}          {EMIT_TOKEN(IDX_PRT_SEL);}
{SEMICOLON}            {EMIT_TOKEN(SEMICOLON);}
{OPEN_BRACKET}         {EMIT_TOKEN(OPEN_BRACKET);}
{CLOSE_BRACKET}        {EMIT_TOKEN(CLOSE_BRACKET);}
{OPEN_SQ_BRACKET}      {EMIT_TOKEN(OPEN_SQ_BRACKET);}
{CLOSE_SQ_BRACKET}     {EMIT_TOKEN(CLOSE_SQ_BRACKET);}
{OPEN_SQ_BRACE}        {EMIT_TOKEN(OPEN_SQ_BRACE);}
{CLOSE_SQ_BRACE}       {EMIT_TOKEN(CLOSE_SQ_BRACE);}
{STAR}                 { yylval.operator=OPERATOR_STAR   ;  EMIT_TOKEN(STAR);}
{PLUS}                 { yylval.operator=OPERATOR_PLUS   ;  EMIT_TOKEN(PLUS);}
{MINUS}                { yylval.operator=OPERATOR_MINUS  ;  EMIT_TOKEN(MINUS);}
{ASL}                  { yylval.operator=OPERATOR_ASL    ;  EMIT_TOKEN(ASL);}
{ASR}                  { yylval.operator=OPERATOR_ASR    ;  EMIT_TOKEN(ASR);}
{LSL}                  { yylval.operator=OPERATOR_LSL    ;  EMIT_TOKEN(LSL);}
{LSR}                  { yylval.operator=OPERATOR_LSR    ;  EMIT_TOKEN(LSR);}
{DIV}                  { yylval.operator=OPERATOR_DIV    ;  EMIT_TOKEN(DIV);}
{POW}                  { yylval.operator=OPERATOR_POW    ;  EMIT_TOKEN(POW);}
{MOD}                  { yylval.operator=OPERATOR_MOD    ;  EMIT_TOKEN(MOD);}
{GTE}                  { yylval.operator=OPERATOR_GTE    ;  EMIT_TOKEN(GTE);}
{LTE}                  { yylval.operator=OPERATOR_LTE    ;  EMIT_TOKEN(LTE);}
{GT}                   { yylval.operator=OPERATOR_GT     ;  EMIT_TOKEN(GT);}
{LT}                   { yylval.operator=OPERATOR_LT     ;  EMIT_TOKEN(LT);}
{L_NEG}                { yylval.operator=OPERATOR_L_NEG  ;  EMIT_TOKEN(L_NEG);}
{L_AND}                { yylval.operator=OPERATOR_L_AND  ;  EMIT_TOKEN(L_AND);}
{L_OR}                 { yylval.operator=OPERATOR_L_OR   ;  EMIT_TOKEN(L_OR);}
{C_EQ}                 { yylval.operator=OPERATOR_C_EQ   ;  EMIT_TOKEN(C_EQ);}
{L_EQ}                 { yylval.operator=OPERATOR_L_EQ   ;  EMIT_TOKEN(L_EQ);}
{C_NEQ}                { yylval.operator=OPERATOR_C_NEQ  ;  EMIT_TOKEN(C_NEQ);}
{L_NEQ}                { yylval.operator=OPERATOR_L_NEQ  ;  EMIT_TOKEN(L_NEQ);}
{B_NEG}                { yylval.operator=OPERATOR_B_NEG  ;  EMIT_TOKEN(B_NEG);}
{B_AND}                { yylval.operator=OPERATOR_B_AND  ;  EMIT_TOKEN(B_AND);}
{B_OR}                 { yylval.operator=OPERATOR_B_OR   ;  EMIT_TOKEN(B_OR);}
{B_XOR}                { yylval.operator=OPERATOR_B_XOR  ;  EMIT_TOKEN(B_XOR);}
{B_EQU}                { yylval.operator=OPERATOR_B_EQU  ;  EMIT_TOKEN(B_EQU);}
{B_NAND}               { yylval.operator=OPERATOR_B_NAND ;  EMIT_TOKEN(B_NAND);}
{B_NOR}                { yylval.operator=OPERATOR_B_NOR  ;  EMIT_TOKEN(B_NOR);}
{TERNARY}              { yylval.operator=OPERATOR_TERNARY;  EMIT_TOKEN(TERNARY);}

{BASE_DECIMAL}         {BEGIN(in_dec_val); EMIT_TOKEN(DEC_BASE);}
{BASE_HEX}             {BEGIN(in_hex_val); EMIT_TOKEN(HEX_BASE);}
{BASE_OCTAL}           {BEGIN(in_oct_val); EMIT_TOKEN(OCT_BASE);}
{BASE_BINARY}          {BEGIN(in_bin_val); EMIT_TOKEN(BIN_BASE);}

<in_bin_val>{BIN_VALUE} {BEGIN(INITIAL); yylval.string = _strdup(yytext); EMIT_TOKEN(BIN_VALUE);}
<in_oct_val>{OCT_VALUE} {BEGIN(INITIAL); yylval.string = _strdup(yytext); EMIT_TOKEN(OCT_VALUE);}
<in_hex_val>{HEX_VALUE} {BEGIN(INITIAL); yylval.string = _strdup(yytext); EMIT_TOKEN(HEX_VALUE);}
<in_dec_val>{DEC_VALUE} {BEGIN(INITIAL); yylval.string = _strdup(yytext); EMIT_TOKEN(DEC_VALUE);}


{NUM_REAL}             {yylval.string=_strdup(yytext);EMIT_TOKEN(NUM_REAL);}
{NUM_UNSIGNED}         {
    yylval.string=_strdup(yytext);
    EMIT_TOKEN(UNSIGNED_NUMBER);
	}

{SYSTEM_ID}            {
    /*yylval.identifier = ast_new_identifier(yytext,yylineno);*/
	yylval.string=_strdup(yytext); 
    EMIT_TOKEN(SYSTEM_ID);
}
{ESCAPED_ID}           {
    /*yylval.identifier = ast_new_identifier(yytext,yylineno);*/
	yylval.string=_strdup(yytext);
    EMIT_TOKEN(ESCAPED_ID);
}
{SIMPLE_ID}            {
    /*yylval.identifier = ast_new_identifier(yytext,yylineno);*/
	yylval.string=_strdup(yytext);
	if (objectCall0(preprocess, SymbolEmitEnabled)) {
		int id = verilog_find_reserved_word(yytext);
		if (id == -1) {
			EMIT_TOKEN(SIMPLE_ID);
		} else {
			EMIT_TOKEN(id);
		}
	}
}

{STRING}               {yylval.string= _strdup(yytext);EMIT_TOKEN(STRING);}

<*>{NEWLINE}              {/*EMIT_TOKEN(NEWLINE); IGNORE */   }
<*>{SPACE}                {/*EMIT_TOKEN(SPACE);   IGNORE */   }
<*>{TAB}                  {/*EMIT_TOKEN(TAB);     IGNORE */   }

<<EOF>> {

    yypop_buffer_state();

    if ( !YY_CURRENT_BUFFER )
    {
        yyterminate();
    }
    else
    {
        YY_BUFFER_STATE cur = YY_CURRENT_BUFFER;
        yylineno = cur -> yy_bs_lineno;
    }
}

.                      {
    EMIT_TOKEN(ANY);
}

%%
